@@ -53,7 +53,10 @@ read_tdf.trimcommand <- function(x,...){
 
 
 # workhorse function for the S3 interfaces
-tdfread <- function(file, missing, weight, ncovars, labels){
+tdfread <- function(file, missing, weight, ncovars, labels) {
+  # First check if the file does indeed exist
+  if (!file.exists(file)) stop(sprintf("Could not find trim data file %s",file), call.=FALSE)
+
   if ( ncovars > 0 && length(labels) == 0 ){
     labels <- paste0("cov",seq_len(ncovars))
   } else if ( ncovars != length(labels)) {
